//Paradise Paintball Javascript File for Dashboard Widget
//History:
// 2009/03/23 - 2.03f3 - bcowan - 
//   - Adding setActive upon loader screen as well
//   - Waiting to download widget once Unity is downloaded 
//Version: 2009/03/03-1-bcowan

//Internal Variables
var totLen = 0;                 //Total length of remote file that is to be downloaded
var finishTracking = false;     //Flag to stop the download tracking bar
var offlineTimeout = false;     //Flag on whether there was no response from the server (it appears we are offline)
var fileName = "dummy";         //Default latest widget filename
var oldCurl = null;             //Remote to the curl command
var timeoutLen = 20000;         //How long before timing out main curl download
var timeoutLeft = timeoutLen;   //
var gnewVer;                    //
var isUnityRunning = false;


//use the varibale and pass it to the unity object. Check there, if the container is out of date. Trigger update routine if needed.
var ContainerVersion = "2.7";

//Parameter set for the webservice { version | build type | platform }
//buildtype: 0 = live, 1 = alpha, 2 = beta
//platform: 0 = cmune, 1 = facebook, 2 = widget
var buildType = "|0|2";

//save the name of the gameobject, that handles all javascript interface calls. Set by unity at startup, calling SetWebInterface(string)
var UnityInterface = "";

function SetWebInterface( gameobjectName )
{
    alert("SetWebInterface " + gameobjectName);

    UnityInterface = gameobjectName;
}

function GetWidgetContainerVersion()
{
    alert("GetWidgetContainerVersion " + ContainerVersion);
    
    document.getElementById("UnityEmbed").SendMessage(UnityInterface, "GetWidgetContainerVersion", ContainerVersion);
}


//This generates the unity source to display and play paintball.unity3d
function genFrontUnity()
{
document.getElementById('stopDiv').style.display='block';
document.getElementById('testDiv').innerHTML=
    '<embed id="UnityEmbed" class="button" src="paintball.unity3d" type="application/vnd.unity" width=760 height=452' 
    + ' imagesignatures="images/images.sig" backgroundimage="images/background.png" progressbarimage="images/_progressbar.png" progressframeimage="images/_progressbg.png"'
	+ ' logoimage="images/unityloading.png"></embed>';
    
    setTimeout ('UnityDashboardPlugin.setActive(true);UnityDashboardPlugin.updateFocus();', .1); 
    
    myAlert("isUnityRunning = TRUE");
    isUnityRunning = true;
}

//This generates the screen that says <play> that when it is clicked will start the game
//It will also display the normal set of tools (clearing the download bar)
function genFront()
{
    document.getElementById('testDiv').innerHTML='<img src="images/PPWidgetFront.jpg" style="cursor:pointer;" onclick="genFrontUnity();"/>';
    showNormalTools();
    document.getElementById('stopDiv').style.display='none';
    
    myAlert("isUnityRunning = FALSE");
    isUnityRunning = false;
}

//This returns the length in bytes of either the final paintball widget if ff=true,
//or the currently download widget if false
//It returns -1 if the file does not exist
function fileLen(ff)
{
    var sln=widget.system('/bin/ls -l',null).outputString;
    var ln=(ff ? sln.match(/([0-9]{3,13}).*paintball.unity3d/) : sln.match(/([0-9]{3,13}).*paintball.pc.unity3d/));
    if(ln && ln.length>1){
      return Number(ln[1]);
    } else {
      return -1;
    }
}

//This displays the progress bar on bottom, it recursively calls itself every 400ms until
//The global variable finishTracking is set to false. Set restart==true to set finishTracking to false
function trackDownload(restart)
{
    if(restart) 
    {
        myAlert('trackDownload: restarted');
        finishTracking=false;
        timeoutLeft=timeoutLen;
        lastLn=0;
    }
    
    if(!finishTracking) 
    {
        var fln = Number(fileLen(false)) + 2;
        var plen = (fln/totLen) * 720;
        
          ihtml=
            '<div class="outerCountDiv" style="position:absolute;width:760px;height:48px;background:url(images/WidgetTabBar_Update.png) blue"><div style="background:rgb(98,98,98);position:absolute;left:20px;top:14px;height:8px;width:'+plen+'px;font-size:6px;">&nbsp;</div></div>';

          var e=document.getElementById('toolsDiv');
          if(!e)
          { return; }
          
          e.innerHTML=ihtml;
          
          if(lastLn == fln) 
          {
            timeoutLeft -= 400;
          } 
          else 
          {
            timeoutLeft = timeoutLen;
          }
          
          lastLn=fln;
          if(timeoutLeft<=0) 
          {
            myAlert('trackDownload: ' + 'reset ' + gnewVer);
            oldCurl.cancel();
            runCurl(gnewVer);
          } 
          else 
          {
            //myAlert('trackDownload: ' + gnewVer + ' left ' + timeoutLeft + ', ' + fln + ', ' + plen);
            setTimeout('trackDownload()',400);
          }
        
    } 
}

//This returns wether the md5sum of paintball.pc.unity3d == http://paradisepaintball.cmune.com/applewidget/md5.aspx
function checkMd5()
{
  os1=widget.system('/usr/bin/curl http://client.cloud.cmune.com/ParadisePaintball/AppleDashboardWidget/'+fileName+'.md5',null).outputString.match(/^([0-9a-fA-F]{32})/)[1];
  os2=widget.system('/sbin/md5 paintball.pc.unity3d',null).outputString.match(/= ([0-9a-fA-F]{32})/)[1];
  myAlert('AA'+os1+ 'EE - AA'+os2+'EE '+(os1==os2));
  return (os1==os2);
}

function myAlert(message)
{
    //alert(message);
    
    //if(widget)
    //    widget.system('echo :' + message + ' >> /ppdebug', null);
}

//This moves the downloaded file to overwrite the play file
function moveToFinal()
{
  widget.system('/bin/mv paintball.pc.unity3d paintball.unity3d');
}

//This runs curl in the background to download http://client.cloud.cmune.com/ParadisePaintball/AppleDashboardWidget/ParadisePaintball{XXX}.unity3d
//It will also display widget.unity3d while it is downloading, and start trackDownload to display the progress bar and track progress of the download
//Once the download has finished it will check the MD5 sum, and on success copy the file over and display the screen to be able to start the game, 
//as well as setting the preference key to the current version id that is in the webservice. (upon failing it will

function runCurl(newVer)
{
    myAlert('runCurl');

  if(fileLen(false) > 0) 
  {
    widget.system('/bin/rm paintball.pc.unity3d');
  }
  
  oldCurl = widget.system('/usr/bin/curl -o paintball.pc.unity3d http://client.cloud.cmune.com/ParadisePaintball/AppleDashboardWidget/' + fileName + '.unity3d', 
  function() 
  {
      oldCurl = null;
      
      if(true) //if(checkMd5())
      {
        moveToFinal();

        noRegen = false;
        finishTracking = true;
        totLen = Number(fileLen(true));

        myAlert('SET CURRENT VERSION TO: ' + newVer + ' with ' + totLen);
        widget.setPreferenceForKey(newVer,'currentVersion');
        widget.setPreferenceForKey(totLen,'currentVersionSize');
                
        genFront();
      } 
      else 
      {
        document.getElementById('testDiv').innerHTML=
            '<table><tr><td align="center" valign="middle" style="color:white">Error, integrity check failed. There could be a problem in your network connection. Download of new version unsuccesful.</td></tr></table>';
        widget.system('/bin/rm paintball.pc.unity3d');
      }
  });
  
  gnewVer = newVer;
  
  setTimeout('trackDownload(true)',3500);
  
  
//  document.getElementById('testDiv').innerHTML='<embed id="UnityEmbed" class="button" src="widget.unity3d"   type="application/vnd.unity" width=760 height=452'
//							+ ' imagesignatures="images/images.sig" backgroundimage="images/background.png" progressbarimage="images/_progressbar.png" //progressframeimage="images/_progressbg.png"'
//							+ ' logoimage="images/unityloading.png"></embed>';  
document.getElementById('testDiv').innerHTML='<embed id="UnityEmbed" class="button" src="./images/PPUpdatingWidget.jpg"  style="cursor:pointer;" width=760 height=452'
							+ ' imagesignatures="images/images.sig" backgroundimage="images/background.png" progressbarimage="images/_progressbar.png" progressframeimage="images/_progressbg.png"'
							+ ' logoimage="images/unityloading.png"></embed>';
                            
                           
  setTimeout ('UnityDashboardPlugin.setActive(true);UnityDashboardPlugin.updateFocus();', .1); 
  
  return true;
}

//This starts curl up in the background
function downloadNewVersion(newVer)
{
    myAlert('Downloading new version ' + newVer);
    
    if(runCurl(newVer)) 
    {
        return true;
    }
}

//This checks to see whether there is a new version of the unity3d widget to download
//it will also check the length of the remote file to see if it is different from the current file
//also, a timeout is set 8 seconds in the future to check whether we are offline
function checkVer()
{
    if(!detectUnityPluginInstalled()) 
    {
        setTimeout('checkVer();',100);
        return;
    }
    
    offlineTimeout=false;
    var url = "http://ws.paradisepaintball.cmune.com/Cmune.svc/AppVersion";
    var params = "message = " + widget.preferenceForKey('currentVersion') + buildType;
    
    http = new XMLHttpRequest()
    
    http.open("POST", url, true);

    //Send the proper header information along with the request
    http.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    http.setRequestHeader("Content-length", params.length);
    //http.setRequestHeader("Connection", "close");
    
    //It will attempt to use totLen, if totLen isn't found then it won't try downloading the new version either.

    myAlert('Webservice Parameter: ' + params);
    
    http.onreadystatechange = function() {//Call a function when the state changes.
        if(!offlineTimeout) 
        {
            if(http.readyState == 4 ) 
            {
                if(http.status == 200) 
                {
                    //myAlert('webservice response: ' + http.responseText);
                
                    var rtxt=http.responseText;
                    var lver=rtxt.match(/GV.*?V="(.*?)"/)[1];
                    fileName=rtxt.match(/GV.*?F="(.*?)"/)[1];
                    
                    myAlert('filename: ' + fileName);
                    
                    totLen = widget.preferenceForKey('currentVersionSize');
                    
                    if(totLen == 0) 
                    {
                        //myAlert('totLen == 0');
                        var s = widget.system('/usr/bin/curl --head http://client.cloud.cmune.com/ParadisePaintball/AppleDashboardWidget/' + fileName + '.unity3d', null).outputString;
                        if(s)
                        {
                            //myAlert('s');
                            var lm = s.match(/Content-Length: ([0-9]+)/);
                            if(lm) 
                            {
                               totLen = Number(lm[1]);
                               widget.setPreferenceForKey(totLen, 'currentVersionSize');
                               //myAlert('totLen = ' + totLen + ',' + widget.preferenceForKey('currentVersionSize'));
                            }
                            else{myAlert('ERROR: no lm');}
                        }
                        else{myAlert('ERROR: no s');}
                    }
                    else
                    {
                        myAlert('current version size: totLen != 0 -- ' + totLen + ',' + widget.preferenceForKey('currentVersionSize'));
                    }
                    
                    cver = widget.preferenceForKey('currentVersion');
                    //myAlert('Current Version ' + cver);
                    
                    myAlert('Current file size: '+ totLen);
                    myAlert('Expected file size: '+ (Number(fileLen(true))));
                    myAlert('Curent Version: '+ cver);
                    myAlert('Expected Version: '+ lver);
                                                                                                    
                    //myAlert('full term: '+ (Number(fileLen(true))) + ' NOT ' + totLen + ' OR (' + totLen + ' NOT 0 AND ' + cver + ' NOT ' + lver + ')');
                    //myAlert('!noRegen: '+ !noRegen);
                    
                    if(Number(fileLen(true)) != totLen || (totLen != 0 && cver != lver)) 
                    {
                        myAlert('Version DEPRECATED');
                        downloadNewVersion(lver);
                    } 
                    else if(!noRegen) 
                    {
                        myAlert('Version OK');
                        genFront();
                        //noRegen = true;
                    }
                    else
                    {
                        myAlert('Version (unhandled case)');
                    }
                } 
                else 
                {
                    myAlert('Error response ' + http.responseText);
                    //Error downloading page
                    if(Number(fileLen(true))==-1) {
                        document.getElementById('testDiv').innerHTML='<table><tr><td align="center" valign="middle" height="477" width="760" style="color:white">There was an error contacting the server, please wait a while to continue downloading the update. The widget will reattempt in 8 seconds.</td></tr></table>';
                        setTimeout("checkVer();",8000);
                    } else if(!noRegen) {
                        genFront();
                        noRegen=true;
                    }
                }
                
                //myAlert('END');
                offlineTimeout=true;
            }
        } 
    }
    
    document.getElementById('testDiv').innerHTML='<embed id="UnityEmbed" class="button" src="./images/PPUpdatingWidget.jpg"  style="cursor:pointer;" width=770 height=452'
							+ ' imagesignatures="images/images.sig" backgroundimage="images/background.png" progressbarimage="images/_progressbar.png" progressframeimage="images/_progressbg.png"'
							+ ' logoimage="images/unityloading.png"></embed>';

    //document.getElementById('testDiv').innerHTML='<table><tr><td align="center" valign="middle" height="477" width="760" style="color:white">...checking server for updates...</td></tr></table>';
    
    setTimeout("checkOfflineTimeout();",8000);
    http.send(params);
}
//This timeout is run 8 seconds after the download server is attempted to be contacted. If the server was not contacted then it assumes we are offline and it checks to see if a paintball.unity3d file is available, if so it is used to generate the screen to begin playing, if not it will try again to check for a version 8 seconds later.
function checkOfflineTimeout()
{
  if(!offlineTimeout) {
    if(Number(fileLen(true))==-1) {
        document.getElementById('testDiv').innerHTML='<table><tr><td align="center" valign="middle" height="465" style="color:white">You are currently offline, please log on to the internet to install the current version of Paradise Paintball. The widget will attempt to download the file again every 16 seconds.</td></tr></table>';
        setTimeout('checkVer();',8000);
    } else if(!noRegen) {
        genFront();
        noRegen=true;
    }
    offlineTimeout=true;
  } 
}

//This is what is run when the Close Game button is run
function doStopGame()
{
    genFront();
}

//This shows the normal toolbar (currently ['Community Website','Play on Facebook','Close Game']);
function showNormalTools()
{
    document.getElementById('toolsDiv').innerHTML=
        '<div class="ppgmi" style="position:absolute;left:20px;" onclick="goToUrl(\'http://paradisepaintball.cmune.com/BuyItem.aspx\');">&nbsp;</div>'
        + '<div class="ppfb" style="position:absolute;left:132px;" onclick="goToUrl(\'http://apps.facebook.com/paradisepaintball\');">&nbsp;</div>'
        + '<div class="pphs" style="position:absolute;left:245px;" onclick="goToUrl(\'http://paradisepaintball.cmune.com/Community.aspx\');">&nbsp;</div>'
        + '<div class="ppcg" style="position:absolute;left:632px;" style="display:none" id="stopDiv" onclick="doStopGame();">&nbsp;</div>';
}

